
"""
Movie recommender system as a showcase project
"""

import argparse
import collections
import random
import os
import pandas as pd
import tensorflow as tf

def load_movielense():
    """Load an return movies and ratings as dataframes"""
    ratings_filename = "data/MovieLense20M/rating.csv"
    ratings_df = pd.read_csv(ratings_filename)

    movies_filename = "data/MovieLense20M/movie.csv"
    movies_df = pd.read_csv(movies_filename)

    return ratings_df, movies_df


def collect_user_histories(ratings, min_rating=2.1):
    """Create a per-user rating list"""
    user_histories = collections.defaultdict(lambda: [])  # dict mapping ids to movies

    ratings = ratings.sort_values(by=['userId', 'timestamp'])

    for user_id, movie_id, rating, _ in ratings.values:
        if rating >= min_rating:
            user_histories[user_id].append(movie_id)

    return user_histories


def create_single_user_examples(user_history, min_len=3, max_len=100):
    """Create examples by sliding a 3-100 window over the history"""
    examples = []
    for label_idx in range(min_len, len(user_history)):
        start_idx = max(0, label_idx - max_len)
        context_movie_ids = user_history[start_idx:label_idx]
        if len(context_movie_ids) < min_len:
            continue

        movie_id = user_history[label_idx]

        feature = {
            "context_movie_id": tf.train.Feature(
                int64_list=tf.train.Int64List(value=context_movie_ids)
             ),
             "label_movie_id": tf.train.Feature(
                int64_list=tf.train.Int64List(value=[movie_id])
             )
        }
        example = tf.train.Example(features=tf.train.Features(feature=feature))
        examples.append(example)

    return examples


def create_user_examples(user_histories, frac=0.8, random_seed=20220827, **kwds):
    """Create user examples for each user history and return train,test lists"""
    examples = []
    progress_bar = tf.keras.utils.Progbar(len(user_histories))
    for user_history in user_histories.values():
        single_user_examples = create_single_user_examples(
            user_history, **kwds
        )
        examples.extend(single_user_examples)
        progress_bar.add(1)

    random.seed(random_seed)
    random.shuffle(examples)

    last_train_index = int(len(examples) * frac)
    train_examples = examples[:last_train_index]
    test_examples = examples[last_train_index:]
    return train_examples, test_examples


def write_user_examples(examples, filename):
    """Write examples to rt Record"""
    progress_bar = tf.keras.utils.Progbar(len(examples))
    with tf.io.TFRecordWriter(filename) as record_writer:
        for example in examples:
            record_writer.write(example.SerializeToString())
            progress_bar.add(1)


def prepare_dataset(args):
    """Read raw CSV files and write TF Record"""
    ratings_df, _ = load_movielense()
    if args.debug:
        ratings_df = ratings_df[:10000]

    user_histories = collect_user_histories(ratings_df)
    train_examples, test_examples = create_user_examples(user_histories)

    train_file = os.path.join(args.output, "train.tfrecord")
    write_user_examples(train_examples, train_file)
    n_train = len(train_examples)
    print(f"File {train_file} with {n_train:d} records created.")

    test_file = os.path.join(args.output, "test.tfrecord")
    write_user_examples(test_examples, test_file)
    n_test = len(test_examples)
    print(f"File {test_file} with {n_test:d} records created.")


commands = {
    "prepare": prepare_dataset,
}

def get_default_parser():
    """Return the default command-line args parser"""
    parser = argparse.ArgumentParser()
    parser.add_argument("command", metavar="CMD", choices=commands,
                        help="Operation to execute")
    parser.add_argument("-i", "--input", metavar="PATH",
                        help="Path to input file")
    parser.add_argument("-o", "--output", metavar="PATH",
                        help="Path to output file(s)")
    parser.add_argument("--debug", default=False, action="store_true",
                        help="Limit test data size")

    return parser


def cli_main(args):
    """Handle CLI args and call requested command"""
    commands[args.command](args)


if __name__ == "__main__":
    parser_ = get_default_parser()
    args_ = parser_.parse_args()
    cli_main(args_)
