
"""
Test cases for the movie recommender system
"""

import unittest
import pandas as pd
import movies

class LoadTests(unittest.TestCase):
    """Test functions concerned with loading and preparing the dataset"""

    @staticmethod
    def toy_ratings():
        """Return a toy dataframe with ratings"""
        return pd.DataFrame({
            "userId": [1, 1, 2, 2, 1],
            "movieId": [1, 2, 1, 3, 4],
            "rating": [3, 1, 4, 3, 5],
            "timestamp": [1, 2, 7, 5, 4],
        },
        columns=["userId", "movieId", "rating", "timestamp"])


    def test_collect_user_histories(self):
        """Check that ratings are correctly aggregated"""
        rating = self.toy_ratings()
        user_movies = movies.collect_user_histories(rating)

        self.assertEqual(user_movies[1], [1, 4])
        self.assertEqual(user_movies[2], [3, 1])
        self.assertEqual(set(user_movies.keys()), {1, 2})


    def test_collect_user_histories_min_rating(self):
        """Check the min rating threshold"""
        rating = self.toy_ratings()
        user_movies = movies.collect_user_histories(rating, min_rating=5)

        self.assertEqual(user_movies[1], [4])
        self.assertEqual(set(user_movies.keys()), {1})


    def assert_feature(self, example, feature_name, value):
        """Assert the int64_list value of a tf.train.Example feature"""
        self.assertEqual(
            example.features.feature[feature_name].int64_list.value,
            value
        )


    def test_user_examples(self):
        """Test the window sliding with default args"""
        history = [1, 2, 3, 4, 5, 6]

        examples = movies.create_single_user_examples(history)

        # First
        self.assert_feature(examples[0], "context_movie_id", [1, 2, 3])
        self.assert_feature(examples[0], "label_movie_id", [4])

        # Second
        self.assert_feature(examples[1], "context_movie_id", [1, 2, 3, 4])
        self.assert_feature(examples[1], "label_movie_id", [5])

        # Third
        self.assert_feature(examples[2], "context_movie_id", [1, 2, 3, 4, 5])
        self.assert_feature(examples[2], "label_movie_id", [6])

        self.assertEqual(len(examples), 3)


    def test_user_examples_args(self):
        """Test the window sliding with constom args"""
        history = [1, 2, 3, 4, 5]

        examples = movies.create_single_user_examples(history, min_len=2, max_len=3)

        # First
        self.assert_feature(examples[0], "context_movie_id", [1, 2])
        self.assert_feature(examples[0], "label_movie_id", [3])

        # Second
        self.assert_feature(examples[1], "context_movie_id", [1, 2, 3])
        self.assert_feature(examples[1], "label_movie_id", [4])

        # Third
        self.assert_feature(examples[2], "context_movie_id", [2, 3, 4])
        self.assert_feature(examples[2], "label_movie_id", [5])

        self.assertEqual(len(examples), 3)


    def test_user_examples_empty(self):
        """Test the window sliding with constom args"""
        history = [1, 2, 3]
        examples = movies.create_single_user_examples(history)
        self.assertEqual(len(examples), 0)

        history = []
        examples = movies.create_single_user_examples(history)
        self.assertEqual(len(examples), 0)
